@@ -184,57 +184,122 @@ run_query <- function(query, id = get_public_id(), key = get_api_key())
 #'
 #' @name download_data
 #' @param location The directory path in which to store the data
-#' @param tournament The name of the tournament, Default is Nomi and is not case-sensitive. Since at the moment the datasets are same for all tournaments this parameter can be left blank.
+#' @param legacy logical; new dataset.  If \code{legacy = TRUE}, legacy 310 feature .zip dataset downloaded
+#' @param load logical; to unzip / read.csv or read_parquet on data.
+#' @param live_only logical; download only the live .parquet dataset.
 #' @return A list containing the training and tournament data objects
 #' @export
 #' @importFrom lubridate today
+#' @import arrow
 #' @import httr
 #' @importFrom utils unzip
 #' @importFrom utils read.csv
+#' @importFrom utils head
+#' @importFrom data.table setDT
 #' @examples
 #' \dontrun{
 #' ## Directory where data files and prediction files to be saved
 #' ## Put custom directory path or use the current working directory
 #' data_dir <- tempdir()
 #'
-#' ## Download data set for current competition
-#' data <- download_data(data_dir,tournament="Nomi")
+#' ## Download legacy dataset for current competition
+#' data <- download_data(data_dir)
 #' data_train <- data$data_train
 #' data_tournament <- data$data_tournament
+#'
+#' ## Download super massive dataset for current competition
+#' download(dat_dir, legacy = FALSE)
 #' }
-download_data <- function(location = tempdir(),tournament="NOMI")
+download_data <- function(location = tempdir(), legacy = TRUE, load = TRUE, live_only = FALSE)
 {
-	## Match tournament ID
-	tournament_id <- match(tolower(tournament),tolower(c("BERNIE","","","KEN","CHARLES","FRANK","HILLARY","NOMI")))
-	if(is.na(tournament_id)) stop("Tournament Name doesn't match")
+  tournament_id <- 8
 
-	## Get download link
-	download_link_query <- paste0('{dataset(tournament:',tournament_id,')}')
-	query_pass <- run_query(query=download_link_query)
-	download_link <- query_pass$data$dataset
+  round <- as.numeric(current_round()[1])
 
-	message("Downloading Data...\n")
+  if(live_only==TRUE & legacy==TRUE) stop("There is no live only dataset for the legacy data.")
 
-	## Download File
-	filename <- file.path(location, paste0("numerai_dataset_", today(), ".zip"))
-	result <- GET(download_link, write_disk(filename, overwrite = TRUE))
+  if(legacy){
+      ## Get download link
+      download_link_query <- paste0('{dataset(tournament:',tournament_id,') }')
 
-	## Unzip the file
-	unzip(filename,overwrite = TRUE,exdir = location)
-	message("Finished downloading data\nReading dataset now...")
+      query_pass <- run_query(query=download_link_query)
 
-	## Read and return the data set
-	data_train <- read.csv(file.path(location, "numerai_training_data.csv"))
-	data_tournament <- read.csv(file.path(location, "numerai_tournament_data.csv"))
-	return(list(data_train=data_train,data_tournament=data_tournament))
+      download_link <- query_pass$data$dataset
+
+      message("Downloading Data...\n")
+
+      ## Download File
+      filename <- file.path(location, paste0("numerai_dataset_", today(), ".zip"))
+      result <- httr::GET(download_link, write_disk(filename, overwrite = TRUE))
+
+      unzip(filename,overwrite = TRUE,exdir = location, list = FALSE)
+      message("Finished downloading data\nReading dataset now...")
+
+      if(load){
+        ## Read and return the data set
+        data_train <- read.csv(file.path(location, "numerai_training_data.csv"))
+        data_tournament <- read.csv(file.path(location, "numerai_tournament_data.csv"))
+        return(list(data_train=data_train,data_tournament=data_tournament))
+      }
+  } else {
+      file_list = c("numerai_live_data_int8.parquet",
+                    "numerai_training_data_int8.parquet",
+                    "numerai_tournament_data_int8.parquet",
+                    "numerai_validation_data_int8.parquet",
+                    "example_validation_predictions.parquet",
+                    "example_predictions.parquet",
+                    "old_data_new_val.parquet")
+
+
+      if(live_only) file_list = file_list[1]
+
+      for(item in file_list){
+          message(paste0("Downloading Data...",item))
+
+          if(item== "old_data_new_val.parquet") round <- 280
+
+          download_link_query <- paste0('{dataset(filename: "',item,'" , round: ',round,')}')
+
+          query_pass <- run_query(query=download_link_query)
+
+          download_link <- query_pass$data$dataset
+
+          ## Download File
+          filename <- file.path(location, item)
+          result <- httr::GET(download_link, write_disk(filename, overwrite = TRUE))
+
+          if(live_only) end_file <-  head(file_list, 1)  else end_file <- tail(file_list, 1)
+
+          if(load && item == end_file){
+              data_live <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[1])))
+              if(!live_only){
+                  data_train <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[2])))
+                  data_tournament <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[3])))
+                  data_validation <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[4])))
+                  example_validation_predictions <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[5])))
+                  example_predictions <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[6])))
+                  old_data_new_val <- data.table::setDT(arrow::read_parquet(file.path(location, file_list[7])))
+
+                  return(list(data_live = data_live, data_train = data_train, data_tournament = data_tournament,
+                              data_validation = data_validation, example_validation_predictions = example_validation_predictions,
+                              example_predictions = example_predictions, old_data_new_val = old_data_new_val))
+              }
+
+              return(list(data_live = data_live))
+          }
+      }
+  }
 }
 
+
 #' Function to submit the Numerai Tournament predictions
 #'
 #' @name submit_predictions
 #' @param submission The data frame of predictions to submit. This should have two columns named "id" & "prediction"
 #' @param location The location in which to store the predictions
 #' @param tournament The name of the tournament, Default is Nomi and is not case-sensitive
+#' @param legacy logical; if \code{legacy = FALSE}, submitting on super massive dataset, else submitting on old legacy format.
+#' @param diagnostics logical; set \code{diagnostics = TRUE} to  to run diagnostics on your upload.
 #' @param model_id Target model UUID (required for accounts with multiple models)
 #' @param prefix The prefix to use for the submission csv file
 #' @return The submission id for the submission made
@@ -246,94 +311,128 @@ download_data <- function(location = tempdir(),tournament="NOMI")
 #' \dontrun{
 #' submission_id <- submit_predictions(submission_data,tournament="Nomi")
 #' }
-submit_predictions <- function(submission, location = tempdir(), tournament = "Nomi", model_id = NULL, prefix = tournament)
+submit_predictions <- function(submission, location = tempdir(), tournament = "Nomi", legacy = TRUE,
+                               diagnostics = FALSE, model_id = NULL, prefix = tournament)
 {
 
-    if(is.null(model_id)) stop("Must provide a model_id")
+  if(is.null(model_id)) stop("Must provide a model_id")
 
-    ## Read the Trigger ID env variable
-    trigger_id = Sys.getenv("TRIGGER_ID")
-    if (trigger_id == "") {
-        trigger_id = NULL
-    }
+  ## Read the Trigger ID env variable
+  trigger_id = Sys.getenv("TRIGGER_ID")
+  if (trigger_id == "") {
+    trigger_id = NULL
+  }
 
-    ## Match tournament ID
-    tournament_id <- match(tolower(tournament),tolower(c("NOMI","SIGNALS"))) + 7
-    if(is.na(tournament_id)) stop("Tournament Name doesn't match")
+  ## Match tournament ID
+  tournament_id <- match(tolower(tournament),tolower(c("NOMI","SIGNALS"))) + 7
+  if(is.na(tournament_id)) stop("Tournament Name doesn't match")
 
-    if(tournament_id == 8 && !all(names(submission)==c("id","prediction"))) stop("Column names should be id & prediction")
+  if(tournament_id == 8 && !all(names(submission)==c("id","prediction"))) stop("Column names should be id & prediction")
+  if(tournament_id == 8 && legacy) version <- 1 else version <- 2
 
-    ## Write out the file
-    date <- gsub("-","_",Sys.Date())
-    submission_filename <- file.path(location, paste0(prefix, "_submission_", date, ".csv"))
-    write.csv(submission, submission_filename, row.names = FALSE)
+  ## Write out the file
+  date <- gsub("-","_",Sys.Date())
+  submission_filename <- file.path(location, paste0(prefix, "_submission_", date, ".csv"))
+  write.csv(submission, submission_filename, row.names = FALSE)
 
-    ## Get a slot on AWS for our submission
-    if(tournament_id == 8){
+  ## Get a slot on AWS for our submission
+  if(tournament_id == 8){
+    if(!diagnostics){
         aws_slot_query <- paste0('query aws_slot_query {
-							submissionUploadAuth (filename : "',paste0(prefix, "_submission_", date, ".csv"),'",
-							                      tournament: ',tournament_id,',
-							                      modelId:"',model_id,'"){
-								filename,
-								url
-							}
-						}')
+    							submissionUploadAuth (filename : "',paste0(prefix, "_submission_", date, ".csv"),'",
+    							                      tournament: ',tournament_id,',
+    							                      modelId:"',model_id,'"
+    							                      ){
+    								filename,
+    								url
+    							}
+    						}')
     } else {
         aws_slot_query <- paste0('query aws_slot_query {
+    							diagnosticsUploadAuth (filename : "',paste0(prefix, "_submission_", date, ".csv"),'",
+    							                       tournament: ',tournament_id,',
+    							                       modelId:"',model_id,'"
+    							                       ){
+    								filename,
+    								url
+    							}
+    						}')
+
+    }
+  } else {
+    aws_slot_query <- paste0('query aws_slot_query {
 							submissionUploadSignalsAuth (filename : "',paste0(prefix, "_submission_", date, ".csv"),'",
 							                      modelId:"',model_id,'"){
 								filename,
 								url
 							}
 						}')
-    }
+  }
 
-    query_pass <- run_query(query=aws_slot_query)
+  query_pass <- run_query(query=aws_slot_query)
 
-    if(tournament_id == 8) numerai_aws_url = query_pass$data$submissionUploadAuth$url else numerai_aws_url = query_pass$data$submissionUploadSignalsAuth$url
+  if(tournament_id == 8){
+    if(!diagnostics) numerai_aws_url = query_pass$data$submissionUploadAuth$url else numerai_aws_url = query_pass$data$diagnosticsUploadAuth$url
+  } else numerai_aws_url = query_pass$data$submissionUploadSignalsAuth$url
 
 
-    ## Upload the predictions
-    mysubmission <- httr::PUT(
-        url = numerai_aws_url,
-        body = upload_file(path = submission_filename)
-    )
+  ## Upload the predictions
+  mysubmission <- httr::PUT(
+    url = numerai_aws_url,
+    body = upload_file(path = submission_filename)
+  )
 
-    if (mysubmission$status_code != 200) {
-        stop(paste0("Uploading submission failed with status code ", mysubmission$status_code))
-    }
+  if (mysubmission$status_code != 200) {
+    stop(paste0("Uploading submission failed with status code ", mysubmission$status_code))
+  }
 
-    ## Register our submission and get evaluation for it
-    if(tournament_id == 8){
+  ## Register our submission and get evaluation for it
+  if(tournament_id == 8){
+    if(!diagnostics){
         register_submission_query <- paste0(
-            'mutation register_submission_query {
-					      							createSubmission (filename : "',query_pass$data$submissionUploadAuth$filename,'",
-					      							tournament: ',tournament_id,',
-					      							triggerId: "',trigger_id,'",
-					      							modelId:"',model_id,'"){id}
-					      						}'
+          'mutation register_submission_query {
+					         							createSubmission (filename : "',query_pass$data$submissionUploadAuth$filename,'",
+					          							tournament: ',tournament_id,',
+					         	  						version: ',version,',
+					         	  						source: "rnumerai",
+					         		  					triggerId: "',trigger_id,'",
+					          							modelId:"',model_id,'"){id}
+					      			    			}'
         )
     } else {
-        register_submission_query <- paste0(
-            'mutation register_submission_query {
+      register_submission_query <- paste0(
+        'mutation register_submission_query {
+					         							createDiagnostics (filename : "',query_pass$data$diagnosticsUploadAuth$filename,'",
+					          							tournament: ',tournament_id,',
+					          							modelId:"',model_id,'"){id}
+					      			    			}'
+      )
+
+    }
+  } else {
+    register_submission_query <- paste0(
+      'mutation register_submission_query {
 					      							createSignalsSubmission (filename : "',query_pass$data$submissionUploadSignalsAuth$filename,'",
 					      							triggerId: "',trigger_id,'",
+					      							source: "rnumerai",
 					      							modelId:"',model_id,'"){id}
 					      						}'
-        )
-    }
+    )
+  }
 
-    query_pass <- run_query(query=register_submission_query)
+  query_pass <- run_query(query=register_submission_query)
 
-    if(tournament_id == 8) query_id <- query_pass$data$createSubmission$id else query_id <- query_pass$data$createSignalsSubmission$id
+  if(tournament_id == 8){
+    if(!diagnostics) query_id <- query_pass$data$createSubmission$id else query_id <- query_pass$data$createDiagnostics$id
+  } else query_id <- query_pass$data$createSignalsSubmission$id
 
-    ## If error
-    if(!is.null(query_pass$errors[[1]]$message)) stop(query_pass$errors[[1]]$message)
+  ## If error
+  if(!is.null(query_pass$errors[[1]]$message)) stop(query_pass$errors[[1]]$message)
 
 
-    ## Return submission id
-    message(paste("Submitted Prediction with id", query_id))
-    return(query_id)
+  ## Return submission id
+  if(!diagnostics) message(paste("Submitted Prediction with id", query_id)) else message(paste("Submitted Diagnostics with id", query_id))
+  return(query_id)
 }
 
 
